# -*- python -*-

import cgi
from _SitePage import *

class overriding(SitePage):

    title = 'Overriding HTMLPage Methods'

    def write_content(self):
        self.writeln(expand_codes(overview))





overview = DIV(

    '''As discussed previously, most servlets will subclass
    C{HTMLPage} and therefore have C{write_html} called as part, if
    not all, of the response to a request.  Here we will explore this
    method.''',
    
    P('It is implemented, simply, as follows: '),

    PRE(cgi.escape("""
def write_html(self):
    self.write_doctype()
    self.writeln('<html>')
    self.write_head()
    self.write_body()
    self.writeln('</html>')
    return True"""),
        klass='codesample'),

    P(cgi.escape('''If you are familiar with the HTML specification
                 you will recognize that this method writes a
                 well-formed HTML document: the declaration of the
                 document type (E{<!DOCTYPE...>} element) followed by
                 the opening E{<HTML>} tag; the HEAD element
                 (E{<HEAD>...</HEAD>}) followed by the BODY element
                 (E{<BODY>...</BODY>}); lastly, the closing E{</HTML>}
                 tag.  You should note that the method returns C{True}
                 which lets the handler know a response was
                 generated.''')),

    P('C{write_doctype} is fully described in the API reference.'),

    P('''C{write_head} calls a series of methods that generate the
      content inside the HEAD element of the document.  While this
      method, and the methods it calls, can be overridden, generally
      you will not need to because the content generated by these
      methods can be controlled by the setting of numerous instance
      variables: S{js}, S{css}, S{css_links}, etc.  Many have been
      demonstrated earlier in this tutorial.  See the API reference
      for complete details.'''),

    P(cgi.escape('''C{write_body} is where the content is generated.
                 Between writing the E{<BODY>} and E{</BODY>} tags,
                 C{write_body} calls C{write_body_parts}.  The base
                 implementation of C{write_body_parts} is implemented
                 as:''')),

    PRE('''
def write_body_parts(self):
    self.write_content()''',
        klass='codesample'),
    
                 
    P('''You may wonder why C{write_body} calls C{write_body_parts} if
      all that method does is call C{write_content}.  Why not have
      C{write_body} call C{write_content} and be done with it?
      Mainly, having a method between C{write_body} and
      C{write_content} makes it much easier to modify page layout.
      '''),

    P('''For example, imagine a company website with a standard page
      layout like this:'''),


    CENTER(TABLE(TR(TD('S{SIDEBAR}',BR(),BR(),BR(),BR(),
                       width='20%'),
                    TD('S{CONTENT}')),
                 frame='box', rules='all', width='50%')),


    P(cgi.escape('''This is a fairly common page layout and is easily
                 implemented in servlets by overriding
                 C{write_body_parts} which, in one possible
                 implementation, can write out a HTML TABLE element
                 and between writing the E{<TD>} and E{</TD>} tags of
                 each cell, call methods to generate the output of the
                 sidebar and content; something like this:''')),

    PRE(cgi.escape("""
def write_body_parts(self):

    self.writeln('<table width="100%" cellpadding="0" cellspacing="0">')
    self.writeln('<tr valign="top"><td width="20%">')
    self.write_sidebar()
    self.writeln('</td><td>')
    self.write_content()
    self.writeln('</td></tr></table>')"""),
        klass='codesample'),


    P('''[Note: I know using tables for page layout is deprecated,
      favoring CSS, but if you have had similar experiences as I, you
      will have noticed relying on CSS for positioning is just not
      quite there yet: too many variances in browsers.  So, I still
      rely on tables. :-( ]''',
      style='font-size:80%;margin:0 .5in 0 .5in'),


    P('''If the above implementation of C{write_body_parts} is placed
      in an abstract servlet and every servlet in the site inherits
      from it, then every page on the site will have the common
      sidebar.'''),

    P('''This is what we have done for this servlet.  It does not
      inherit from C{TutorialBase} like all the other servlets thus
      far, but from another servlet, C{SitePage}, which in turn
      inherits from C{TutorialBase}.  If you view the python source
      you will see that the only method implemented in this servlet is
      C{write_content}.  C{SitePage} (defined in S{_SitePage.py})
      overrides C{write_body_parts} which generates a TABLE and calls
      C{write_sidebar} and C{write_content} in the appropriate cells.
      The sidebar for this E{site} provides links to all the
      tutorials.'''),

    P('''The next servlet will demonstrate the ease of modifying page
      layout either by modifying the implementation in the abstract
      base class or by extending the implementation.''')
    )
