# -*- python -*-

from _TutorialBase import *

class lifecycle(TutorialBase):

      title = 'Request Life-Cycle'
      css_links = ['tutorial.css']

      def write_content(self):
            self.writeln(expand_codes(overview))



overview = DIV(

      P('''The previous servlet introduced the C{auth} method.  That
        method is one of many methods called during the life-cycle of
        a request.  This and the remaining servlets will introduce
        methods that are called during the life-cycle of a request,
        many of which you can override to control processing and page
        layout.  This is where (imho) the real power of servlets come
        into play.'''),

      P('''Below is an anotated listing of the handler code.  It is
        well worth the effort to study the servlet processing that
        happens here.  Understanding what happens at this level will
        help you exploit servlets to their full potential:'''),
      
      PRE('''
 1.    # get the servlet
 2.    servlet = _sm.get_servlet(filename)
 3.
 4.    # stuff request in servlet
 5.    servlet.req = req
 6.
 7.    # call user methods
 8.    try:
 9.        servlet.auth()
10.        servlet.prep()
11.        if not servlet.respond():
12.            return apache.HTTP_NO_CONTENT
13.        servlet.wrapup()
14.
15.    finally:
16.
17.        # handler wrapup of servlet
18.        servlet._finally()
19.
20.        # if the servlet is reusable, put it back into the cache
21.        if servlet.reusable:
22.            _sm.put_servlet(servlet)''',
          klass='codesample'),

      P('''S{Line #2:} C{_sm} is the handler S{servlet manager}.  You
        do not need to know any details about its machinations.  Here,
        the servlet manager, given the the filename of the mps file,
        returns a servlet instance.'''),

      P('''S{Line #5:} The request object is set as an instance
        variable.'''),

      P('''S{Line #8:} A series of methods of C{servlet} are called
        inside a C{try} clause.'''),

      P('''S{Line #9-13:} In order, C{auth}, C{prep}, C{respond} and
        C{wrapup} are called.  ''',

        UL(LI('C{auth} was discussed in the ', A('previous', href='auth2'),
              ' servlet.'),

           LI('''C{prep} is called to do any pre-processing, before
              generating the response.  The base method processes
              C{query_vars} and C{form_vars} here, for instance.  If
              you override C{prep}, you E{must} call the superclass
              method (and typically, as the first action in your
              method).  What do you do in C{prep}?  This is a good
              place to validate incoming data, conditionally set
              instance variables (e.g., you can dynamically set the
              C{js} instance variable to include or not include
              javascript), open database connections, initialize
              variables for the request, etc.  Note that the return
              value is ignored.'''),

           LI('''C{respond} is where the response is generated and
              output written to the client.  Typically, you will not
              need to override this method.  Generating responses will
              be discussed in more detail in the next few servlets.
              Note here, however, that the return value, if false,
              returns C{apache.HTTP_NO_CONTENT}.'''),

           LI('''C{wrapup} is where any necessary post-processing
              occurs.  The base method flushes output to the client.
              If you override C{wrapup}, you E{must} call the
              superclass method (and typically, as the last action in
              your method).  What do you do in C{wrapup}?  Typically,
              not much, if anything.  In most cases you will not
              override this method, but here is the place you may want
              to do any clean-up of external connections,
              conditionally invalidate sessions, etc.  Note that the
              return value is ignored.'''))),
        
      P('''S{Line #15:} The C{finally} clause begins.  The previous
        methods are called inside a C{try...finally...} statement to
        insure proper cleanup in case an exception is raised.  This is
        a good place to mention this: you may raise an exception
        anywhere in your code (C{apache.HTTP_UNAUTHORIZED} in C{auth}
        or after conditionally checking the request in C{prep},
        C{apache.HTTP_BAD_REQUEST}, etc.) and know that the request
        will be cleaned up properly.'''),

      P('''S{Line #18:} C{_finally} is called.  This is where the
        session (if C{use_session} is true) is saved and garbage
        collected, so as to prevent session locks.  Also, any
        unflushed output (due to an exception being raised) is garbage
        collected.'''),

      P('''S{Line #22-22:} C{servlet} is placed back into the
        S{servlet manager} cache if it is marked as reusable (the
        default).'''))

